<?php

class ControllerExtensionPaymentBspb extends Controller
{
    public function index()
    {
        $this->load->language('extension/payment/rbs');
        $data['button_confirm'] = $this->language->get('button_confirm');
        $data['start_url'] = $this->url->link('extension/payment/bspb/start');
        return $this->load->view('extension/payment/bspb', $data);
    }

    public function start()
    {
        $this->load->model('checkout/order');
        $order_id = $this->session->data['order_id'];
        $order_info = $this->model_checkout_order->getOrder($order_id);

        if ($this->config->get('bspb_mode')) {
            $bankUrl = 'https://mpitest.bspb.ru:5443/Exec';
        } else {
            $bankUrl = 'https://mpi.bspb.ru:5443/Exec';
        }

        $xml = new \SimpleXMLElement('<?xml version="1.0" encoding="UTF-8"?><TKKPG></TKKPG>');
        $xmlRequest = $xml->addChild('Request');
        $xmlRequest->addChild('Operation', 'CreateOrder');
        $xmlRequest->addChild('Language', $this->config->get('bspb_language'));
        $xmlOrder = $xmlRequest->addChild('Order');
        $xmlOrder->addChild('OrderType', $this->config->get('bspb_order_type'));
        $xmlOrder->addChild('Merchant', $this->config->get('bspb_merchant'));
        $xmlOrder->addChild('Amount', ((float)$order_info['total'] * 100));
        $xmlOrder->addChild('Currency', $this->config->get('bspb_currency'));
        $xmlOrder->addChild('Description', $this->config->get('bspb_description'));
        
        $resultUrl = $this->getResultUrl();
        $xmlOrder->addChild('ApproveURL', $resultUrl['ApproveURL']);
        $xmlOrder->addChild('CancelURL', $resultUrl['CancelURL']);
        $xmlOrder->addChild('DeclineURL', $resultUrl['DeclineURL']);
        
        $xmlAddParams = $xmlOrder->addChild('AddParams');
        $xmlAddParams->addChild('ShopOrderNum', $order_id);

        $strResponse = $this->sendPostXml($bankUrl, $xml->asXML(), $this->config->get('bspb_file_rem'), $this->config->get('bspb_file_key'));
        if (is_array($strResponse) && strlen($strResponse['error']) == 0) {
            $response = $this->XmlToArray($strResponse['data']);
            if ($response['Response']['Status'] == '00') {
                $urlToBank = $response['Response']['Order']['URL'] . "?ORDERID=" . $response['Response']['Order']['OrderID']."&SESSIONID=" . $response['Response']['Order']['SessionID'];
                $this->response->redirect($urlToBank);
            } else {
                $data['error'] = 'Error: status ' . $response['Response']['Status'];
            }
        } else {
            $data['error'] = 'Error: ' . $strResponse['error'];
        }
        echo $data['error'];
    }

    public function callback()
    {
        if (strlen($_REQUEST['xmlmsg']) > 0) {
            $data = $this->XmlToArray($_REQUEST['xmlmsg']);
            if (array_key_exists('Message', $data)) {
                $orderId = $data['Message']['OrderID'];
                $sessionId = $data['Message']['SessionID'];
            } else {
                $orderId = $data['OrderID'];
                $sessionId = $data['SessionId'];
            }

            if ($this->config->get('bspb_mode')) {
                $bankUrl = 'https://mpitest.bspb.ru:5443/Exec';
            } else {
                $bankUrl = 'https://mpi.bspb.ru:5443/Exec';
            }

            $xml = new \SimpleXMLElement('<?xml version="1.0" encoding="UTF-8"?><TKKPG></TKKPG>');
            $xmlRequest = $xml->addChild('Request');
            $xmlRequest->addChild('Operation', 'GetOrderInformation');
            $xmlRequest->addChild('Language', $this->config->get('bspb_language'));
            $xmlOrder = $xmlRequest->addChild('Order');
            $xmlOrder->addChild('Merchant', $this->config->get('bspb_merchant'));
            $xmlOrder->addChild('OrderID', $orderId);
            $xmlRequest->addChild('SessionID', $sessionId);
            $xmlRequest->addChild('ShowParams', 'true');
            $xmlRequest->addChild('ShowOperations', 'true');
            $xmlRequest->addChild('ClassicView', 'true');

            $strResponse = $this->sendPostXml($bankUrl, $xml->asXML(), $this->config->get('bspb_file_rem'), $this->config->get('bspb_file_key'));

            if (is_array($strResponse) && strlen($strResponse['error']) == 0) {
                $dataInit = $this->XmlToArray($strResponse['data']);
                if ($dataInit && $dataInit['Response']['Status'] == '00') {
                    $this->load->model('checkout/order');
                    $order_id = $this->findOrderId($dataInit['Response']['Order']['row']['OrderParams']['row']);
                    $order_info = $this->model_checkout_order->getOrder($order_id);
                    if ($dataInit['Response']['Order']['row']['Orderstatus'] == 'APPROVED') {
                        if ((float)$order_info['total'] * 100 == (float)$dataInit['Response']['Order']['row']['Amount']) {
                            $this->model_checkout_order->addOrderHistory($order_id, $this->config->get('bspb_order_status_id'));
                            $this->response->redirect($this->url->link('checkout/success', '', true));
                        }
                    }
                }
            }
        }
        $this->response->redirect($this->url->link('checkout/failure', '', true));
    }

    private function sendPostXml($url, $xml, $pathCertPem, $pathCertKey)
    {
        $error = '';
        $ch = curl_init($url);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 1);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
        curl_setopt($ch, CURLOPT_SSLCERT, $pathCertPem);
        curl_setopt($ch, CURLOPT_SSLKEY, $pathCertKey);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $xml);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_VERBOSE, 1);
        if (!$answer = curl_exec($ch)) {
            $error = 'Error with connection: ' . curl_errno($ch);
        }
        curl_close($ch);

        return array(
            'data' => $answer,
            'error' => $error
        );
    }

    private function XmlToArray($strXml)
    {
        $strXml = str_replace("\\\"", "'", $strXml);
        $json = json_encode(simplexml_load_string($strXml));
        return json_decode($json, true);
    }

    private function getResultUrl()
    {
        $url = $this->url->link('extension/payment/bspb/callback');

        return array(
            'ApproveURL' => $url,
            'CancelURL' => $url,
            'DeclineURL' => $url,
        );
    }

    private function findOrderId($arr)
    {
        foreach($arr as $val)
            if (is_array($val) && key_exists('PARAMNAME',$val) && key_exists('VAL',$val) && $val['PARAMNAME'] == 'ShopOrderNum')
                return $val['VAL'];
        return -1;
    }
}
